@@ -61,6 +61,7 @@ struct _GstVaFilter
   GstVideoOrientationMethod orientation;
 
   guint32 scale_method;
+  guint32 interpolation_method;
 
   gboolean crop_enabled;
 
@@ -154,6 +155,8 @@ gst_va_filter_class_init (GstVaFilterClass * klass)
       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
   g_object_class_install_properties (gobject_class, N_PROPERTIES, g_properties);
+
+  gst_type_mark_as_plugin_api (GST_TYPE_VA_INTERPOLATION_METHOD, 0);
 }
 
 static void
@@ -582,6 +585,11 @@ static const struct _CBDesc {
   [VAProcColorBalanceAutoBrightness] =
       { "auto-brightness", "Auto-Brightness", "Enable auto brightness",
         GST_VA_FILTER_PROP_AUTO_BRIGHTNESS    },
+  /**
+   * GstVaPostProc:auto-contrast:
+   *
+   * Since: 1.20
+   */
   [VAProcColorBalanceAutoContrast] =
       { "auto-contrast", "Auto-Contrast", "Enable auto contrast",
         GST_VA_FILTER_PROP_AUTO_CONTRAST },
@@ -689,6 +697,13 @@ gst_va_filter_install_properties (GstVaFilter * self, GObjectClass * klass)
 
   if (self->pipeline_caps.mirror_flags != VA_MIRROR_NONE
       || self->pipeline_caps.rotation_flags != VA_ROTATION_NONE) {
+    /**
+     * GstVaPostProc:video-direction:
+     *
+     * Video direction: rotation and flipping
+     *
+     * Since: 1.20
+     */
     g_object_class_install_property (klass, GST_VA_FILTER_PROP_VIDEO_DIR,
         g_param_spec_enum ("video-direction", "Video Direction",
             "Video direction: rotation and flipping",
@@ -696,6 +711,36 @@ gst_va_filter_install_properties (GstVaFilter * self, GObjectClass * klass)
             common_flags));
   }
 
+  /**
+  * GstVaPostProc:scale-method
+  *
+  * Sets the scale method algorithm to use when resizing.
+  *
+  * Since: 1.22
+  */
+  if (GST_VA_DISPLAY_IS_IMPLEMENTATION (self->display, INTEL_IHD)) {
+    g_object_class_install_property (klass,
+        GST_VA_FILTER_PROP_SCALE_METHOD,
+        g_param_spec_enum ("scale-method", "Scale Method",
+            "Scale method to use",
+            GST_TYPE_VA_SCALE_METHOD, VA_FILTER_SCALING_DEFAULT, common_flags));
+  }
+
+  /**
+  * GstVaPostProc:interpolation-method
+  *
+  * Sets the interpolation method algorithm to use when resizing.
+  *
+  * Since: 1.26
+  */
+  if (GST_VA_DISPLAY_IS_IMPLEMENTATION (self->display, INTEL_IHD)) {
+    g_object_class_install_property (klass,
+        GST_VA_FILTER_PROP_INTERPOLATION_METHOD,
+        g_param_spec_enum ("interpolation-method", "Interpolation Method",
+            "Interpolation method to use for scaling",
+            GST_TYPE_VA_INTERPOLATION_METHOD, VA_FILTER_INTERPOLATION_DEFAULT,
+            common_flags));
+  }
   return TRUE;
 }
 
@@ -930,6 +975,18 @@ gst_va_filter_set_scale_method (GstVaFilter * self, guint32 method)
   return TRUE;
 }
 
+gboolean
+gst_va_filter_set_interpolation_method (GstVaFilter * self, guint32 method)
+{
+  g_return_val_if_fail (GST_IS_VA_FILTER (self), FALSE);
+
+  GST_OBJECT_LOCK (self);
+  self->interpolation_method = method;
+  GST_OBJECT_UNLOCK (self);
+
+  return TRUE;
+}
+
 static gboolean
 _from_video_orientation_method (GstVideoOrientationMethod orientation,
     guint * mirror, guint * rotation)
@@ -1070,7 +1127,8 @@ gst_va_filter_get_caps (GstVaFilter * self)
 
   if (mem_types & VA_SURFACE_ATTRIB_MEM_TYPE_VA) {
     feature_caps = gst_caps_copy (base_caps);
-    features = gst_caps_features_from_string (GST_CAPS_FEATURE_MEMORY_VA);
+    features =
+        gst_caps_features_new_single_static_str (GST_CAPS_FEATURE_MEMORY_VA);
     gst_caps_set_features_simple (feature_caps, features);
     caps = gst_caps_merge (caps, feature_caps);
   }
@@ -1612,7 +1670,7 @@ _create_pipeline_buffer (GstVaFilter * self, GstVaSample * src,
     .output_surface_flag = dst->flags,
     .input_color_properties = self->input_color_properties,
     .output_color_properties = self->output_color_properties,
-    .filter_flags = self->scale_method,
+    .filter_flags = self->scale_method | self->interpolation_method,
     /* output to SDR */
     .output_hdr_metadata = NULL,
   };
@@ -1795,7 +1853,7 @@ gst_va_filter_compose (GstVaFilter * self, GstVaComposeTransaction * tx)
       .surface_region = &sample->input_region,
       .output_region = &sample->output_region,
       .output_background_color = 0xff000000,
-      .filter_flags = self->scale_method,
+      .filter_flags = self->scale_method | self->interpolation_method,
     };
     /* *INDENT-ON* */
     GST_OBJECT_UNLOCK (self);
@@ -1947,3 +2005,26 @@ gst_va_scale_method_get_type (void)
   }
   return type;
 }
+
+GType
+gst_va_interpolation_method_get_type (void)
+{
+  static gsize type = 0;
+  static const GEnumValue values[] = {
+    {VA_FILTER_INTERPOLATION_DEFAULT, "Default interpolation", "default"},
+    {VA_FILTER_INTERPOLATION_NEAREST_NEIGHBOR, "Nearest neighbor interpolation",
+        "nearest-neighbor"},
+    {VA_FILTER_INTERPOLATION_BILINEAR, "Bilinear interpolation", "bilinear"},
+    {VA_FILTER_INTERPOLATION_ADVANCED, "Advanced interpolation method is "
+          "defined by each implementation and usually gives best quality.",
+        "advanced"},
+    {0, NULL, NULL},
+  };
+
+  if (g_once_init_enter (&type)) {
+    const GType _type =
+        g_enum_register_static ("GstVaInterpolationMethod", values);
+    g_once_init_leave (&type, _type);
+  }
+  return type;
+}
